
#include <assert.h>
#include "UmlClass.h"

#include "UmlCom.h"
#include "UmlOperation.h"
#include "UmlRelation.h"
#include "aRelationKind.h"
#include "UmlAttribute.h"
#include "UmlArtifact.h"
#include "UmlClassView.h"
#include "UmlDeploymentView.h"
#include "UmlPackage.h"
#include "UserInput.h"

void UmlClass::genCxxTestCase() {
QCString strProp;
bEnum = false; // should generate enum traits?

// If this class is already a test suite, fixture or global/world fixture, don't create a test for it
if( propertyValue("CxxTestSuite", strProp) ){
	UmlCom::trace("<font color='orange'>  ... skipping existing test class <b>" + name() + "</b></font>");
	return;
}

if( propertyValue("CxxTestGlobalFixture", strProp) ){
	UmlCom::trace("<font color='orange'>  ... skipping existing global test fixture <b>" + name() + "</b></font>");
	return;
}

// show user input dialog
if( ! UserInput::get_instance()->getUserInput() ){
	return;
}

bGenFixture = UserInput::get_instance()->get_bGenFixture();
cv = UserInput::get_instance()->get_selectedClassView();
dv = UserInput::get_instance()->get_selectedDeploymentView();

if( stereotype() == "enum"){
	UmlCom::trace("\nGenerating Enum Traits for enum <b>" + name() + "</b>");
	bEnum = true;
}else{
	UmlCom::trace("\nGenerating Test Case for class <b>" + name() + "</b>");
}

// Get the name of this class and prefix it with "Test" or "EnumTraits"
QCString testClassName;
if(bEnum)
	testClassName = "EnumTraits" + name();
else
	testClassName = "Test" + name();

// ==========================================================================
// Add a new class to the parent with the prefixed name
// ==========================================================================
// Check the parent must be of ClassView type
if (cv->kind() == aClassView){
	UmlClass* newClass = cv->getClass(testClassName);
	if( !newClass ){ // if not found an existing class
		newClass = create(cv, testClassName); // create the new class under parent
		newClass->set_PropertyValue("CxxTestSuite", "1");
		UmlRelation::create(aDependency, newClass, this); //creat a relation
	}

	if(newClass) { // if new class was created
		if(bEnum){
			genEnumTraits(newClass);
		}else{
			newClass->set_CppDecl("${comment}${template}class ${name} : public CxxTest::TestSuite {\n${members}};\n\n${inlines}");
			// Generate test class members
			genTestClassMembers(newClass);
		}

		// ==========================================================================
		// create a source artifact with custom definition under selected deployment view
		// ==========================================================================
		if(dv){
			UmlArtifact* artifact = UmlArtifact::genTestArtifact(dv, newClass);
		} // if (dv)
	}else{ // if (newClass)
		UmlCom::trace(QCString("could not creat new class ") + newClass->name() );
	}
}else{ // if (aClassView)
	UmlCom::trace("Error: The parent of the selected item is not a <i>class view</i>");
}
}

//Generate the test class members
void UmlClass::genTestClassMembers(UmlClass * newClass) {
  unsigned int i;
  QCString _namespacePrefix;
  bool bNewOpCreated = false;
  
  assert(parent()->parent()->kind() == aPackage);
  UmlPackage* pkg = ( UmlPackage* ) parent()->parent();
  _namespacePrefix = pkg->cppNamespace();
  _namespacePrefix = (_namespacePrefix.isEmpty() ? "" : _namespacePrefix + "::");
  
  if(bGenFixture){
  	// Generate an attribute object of the class under test
  	UmlTypeSpec type;
  	type.explicit_type = _namespacePrefix + name() + "*"; // pointer
  			
  	UmlAttribute* attrib = newClass->getAttribute("pTestObj");
  	if(!attrib) attrib = UmlAttribute::create(newClass, "pTestObj" );
  	attrib->set_Visibility(PublicVisibility);
  	attrib->set_Type( type  ); 
  
  	bNewOpCreated = UmlOperation::genTest(newClass, "setUp");
  	// to prevent overwriting an existing operation body,
  	// set operation body only if a new operation was created
  	if (bNewOpCreated ){
  		UmlOperation* pOp = newClass->getOperation("setUp");
  		pOp->set_CppBody("pTestObj = new " + _namespacePrefix + name() + "();\n" );
  	}
  
  	bNewOpCreated = UmlOperation::genTest(newClass, "tearDown");			
  	// to prevent overwriting an existing operation body,
  	// set operation body only if a new operation was created
  	if (bNewOpCreated ){	
  		UmlOperation* pOp = newClass->getOperation("tearDown");
  		pOp->set_CppBody("delete pTestObj;\n" ); // create object
  	}
  }else{
  	// Generate an attribute object of the class under test
  	UmlTypeSpec type;
  	type.explicit_type = _namespacePrefix + name();
  	UmlAttribute* attrib = newClass->getAttribute("testObj");
  	if(!attrib) attrib = UmlAttribute::create(newClass, "testObj" );
  	attrib->set_Visibility(PublicVisibility);
  	attrib->set_Type( type );
  }
  
  // Find the public operations in the selected class and generate test stubs
  QVector<UmlItem> myChildren = children();
  for(i=0; i < myChildren.size(); i++){
  	UmlClassMember* child = (UmlClassMember*)myChildren[i];
  	if(child->kind() == anOperation){
  		child->genTest(newClass); // Generate the test for this operation
  	} // end if (child->kind ...)
  } // end for
}

//Generate the test class members
void UmlClass::genEnumTraits(UmlClass * newClass) {
  unsigned int i;
  if(bEnum){
  	QCString strEnumTraits = "${comment}\nCXXTEST_ENUM_TRAITS( " + name() + ",\n${members});\n";
  	newClass->set_CppDecl( strEnumTraits);
  	
  	// make it a struct so public/protected visibility specifiers are not generate
  	newClass->set_Stereotype( "struct" );
  
  	QVector<UmlItem> myChildren = children();
  	for(i=0; i < myChildren.size(); i++){
  		UmlClassMember* child = (UmlClassMember*)myChildren[i];
  		if(child->kind() == anAttribute){
  			UmlAttribute* attrib = newClass->getAttribute(child->name());
  			if (!attrib){
  				attrib = UmlAttribute::create(newClass, child->name());
  				attrib->set_Visibility( PublicVisibility );
  			}
  			attrib->set_CppDecl( "\tCXXTEST_ENUM_MEMBER( ${name} )" );	
  		} // end if (child->kind ...)
  	} // end for
  }
}

UmlOperation* UmlClass::getOperation(QCString who) {
    const QVector<UmlItem> ch = children();
  
    for (unsigned i = 0; i != ch.size(); i += 1)
      if ((ch[i]->kind() == anOperation) &&
          (ch[i]->name() == who))
        return (UmlOperation *) ch[i];
  
    return 0;
}

UmlAttribute* UmlClass::getAttribute(QCString who) {
    const QVector<UmlItem> ch = children();
  
    for (unsigned i = 0; i != ch.size(); i += 1)
      if ((ch[i]->kind() == anAttribute) &&
          (ch[i]->name() == who))
        return (UmlAttribute *) ch[i];
  
    return 0;
}

